/* 
 * Just Another Team
 */

package model;

import java.util.LinkedList;

public class Car {

	// Varosra mutato referencia
	protected City city;

	// Kovetkezo lepes
	protected int nextMove;

	// Csomopontra mutato referencia
	protected Node node;

	// Sebesseg
	protected int speed;

	// Ero
	protected int strength;

	// Varakozas
	protected int wait;

	// Konstrukor
	public Car(int speed, int strength, City city) {

		// Beallitjuk a megfelelo tagvaltozokat
		this.speed = speed;
		this.strength = strength;
		this.city = city;

		// Kovetkezo lepes meg nem ismert
		nextMove = -1;

		// Varakazasi ido
		wait = speed;

		// Kezdetben ismeretlen a csomopont
		node = null;
	}

	// CallBacket megvalosito, utkozest lekezelo fuggveny
	public void collision(RunnerCar r) {
		r.collisionWithCar(this);
	}

	// Az auto megszunik
	public void destroy() {
		// Toroljuk a varosbol
		city.removeCar(this);

		// Toroljuk a csomopontbol
		if (node != null)
			node.setCar(null);

		// Beallitjuk, hogy nem ismert a csomopont
		node = null;
	}

	// Getter fuggveny belso tagvaltozohoz
	public int getNextMove() {
		return nextMove;
	}

	// Getter fuggveny belso tagvaltozohoz
	public Node getNode() {
		return node;
	}

	// Getter fuggveny
	public int getNodeId() {
		// Megvizsgaljuk, hogy letezik-e a csomopont
		if (node != null)
			return node.getId();
		else
			return -1;
	}

	// Getter fuggveny belso tagvaltozohoz
	public int getSpeed() {
		return speed;
	}

	// Getter fuggveny belso tagvaltozohoz
	public int getStrength() {
		return strength;
	}

	// Getter fuggveny belso tagvaltozohoz
	public int getX() {
		return node.getX();
	}

	// Getter fuggveny belso tagvaltozohoz
	public int getY() {
		return node.getY();
	}

	// Eldonti, hogy behajthatunk-e a csomopontba az elsobbsegi szabalyok
	// figyelembevetelevel
	protected boolean givePriority(Node target) {

		// Megallapitjuk, honnan erkezik az auto
		int inDir = target.getDir(node);

		// Ha nincs jelzolampa
		if (target.getTrafficLight() == -1) {

			// Az elsobbsegi sorrend alapjan ellenorizzuk a bejovo csomopontokat
			LinkedList<Integer> priority = target.getPriority();

			// Minden bejovo csomopontra lefuttatjuk
			for (int i = 0; i < 4 && priority.get(i) != inDir; i++) {
				if (target.getInNodes()[priority.get(i)] != null
						&& target.getInNodes()[priority.get(i)].getCar() != null)
					return false;
			}

			// Ha nincs erkezo auto
			return true;
		} else
			// Ha jelzolampas, akkor a dontes egyszeru
			return (target.getTrafficLight() == inDir);
	}

	// Setter fuggveny belso tagvaltozohoz
	public void setCity(City city) {
		this.city = city;
	}

	// Setter fuggveny belso tagvaltozohoz
	public void setNode(Node n) {
		this.node = n;
	}

	// Auto lepteteset vegzo fuggveny
	public void step() {

		// Ha meg palyan van az auto
		if (node != null) {

			// Ha meg kell varni a kovetkezo lepesig
			if (wait > 0) {
				wait--;
			}

			// Megnezzuk, hogy van-e kivezeto ut
			if (node.activeOutNodes() == 0) {
				this.destroy();
				return;
			}

			// Ha meg nem dontottunk a kimeno irany felol
			if (nextMove == -1) {
				do {
					// Akkor dontunk
					nextMove = Game.random.nextInt(4);
					// Amig nem kapunk jo csomopontot
				} while (node.getOutNodes()[nextMove] == null);
			}

			// Ha mar nem kell tovabb varakozni
			if (wait == 0) {
				// Lekerdezzuk a celcsomopontot
				Node target = node.getOutNodes()[nextMove];

				// Megvizsgaljuk, hogy tovabbhaladhatunk-e
				if (target.getCar() == null && givePriority(target)) {

					// Lepes
					node.setCar(null);
					node.getOutNodes()[nextMove].setCar(this);
					node = node.getOutNodes()[nextMove];
					wait = speed;
					nextMove = -1;

					// Lepunk, hogy eldoljon a kovetkezo lepesunk is a kepernyo
					// frissitese elott
					step();
				}
			}
		}
	}
}
