/* 
 * Just Another Team
 */

package model;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.TreeMap;
import java.util.Map.Entry;

// City osztaly
public class City {

	// Autokat tarolo adatstruktura
	private LinkedList<Car> cars;
	// Gamere mutato belso referencia
	private Game game;
	// Szint
	private int level;
	// Csomopontokat tarolo adatstruktura
	private TreeMap<Integer, Node> nodes;
	// A rablok autojara mutato referencia
	private RunnerCar runnerCar;

	// City objektum
	public City(Game game) {
		// Inicializaljuk az adatstrukturakat
		nodes = new TreeMap<Integer, Node>();
		cars = new LinkedList<Car>();

		// Meg nem ismert a rablok autoja
		runnerCar = null;

		// Beallitjuk a belso referenciat
		this.game = game;

		// A 0-as szint az alapertelmezett
		level = 0;
	}

	// Hozzaad a varoshoz egy csomopontot
	private void addNode(int id, int type, int x, int y, boolean hiding) {
		Node t = new Node(this, id, type, hiding);
		game.getView().createNodeView(t, x, y);
		nodes.put(id, t);
	}

	// Osszekapcsol, ket megadott azonositoju csomopontot
	private void connect(int node1, int node2, int dir1, int dir2) {
		nodes.get(node1).connect(false, dir1, nodes.get(node2));
		nodes.get(node2).connect(true, dir2, nodes.get(node1));
	}

	// Getter fuggveny belso tagvaltozohoz
	public LinkedList<Car> getCars() {
		return cars;
	}

	// Lekerdezzuk az ures csomopontokat a nyul szamara
	private LinkedList<Node> getEmptyNodes() {
		// Letrehozunk egy uj listat
		LinkedList<Node> res = new LinkedList<Node>();
		// Bejarjuk a csompontokat
		for (Entry<Integer, Node> e : nodes.entrySet()) {
			// Ha a csomopont ures, betesszuk
			if (e.getValue().getCar() == null) {
				res.add(e.getValue());
			}
		}
		// Visszaterunk az elkeszitett listaval
		return res;
	}

	// Lekerdezzuk a bejovo csompontokat
	private LinkedList<Node> getEntryNodes() {
		// Letrehozunk egy uj listat
		LinkedList<Node> res = new LinkedList<Node>();

		// Letrehozunk egy uj listat
		for (Entry<Integer, Node> e : nodes.entrySet()) {
			if (e.getValue().activeInNodes() == 0) {
				// Ha a csomopont ures, betesszuk
				res.add(e.getValue());
			}
		}
		return res;
	}

	// Getter fuggveny belso tagvaltozohoz
	public Game getGame() {
		return game;
	}

	// Getter fuggveny belso tagvaltozohoz
	public int getLevel() {
		return level;
	}

	// Getter fuggveny belso tagvaltozohoz
	public TreeMap<Integer, Node> getNode() {
		return nodes;
	}

	// Getter fuggveny belso tagvaltozohoz
	public RunnerCar getRunnerCar() {
		return runnerCar;
	}

	// Inicializalja a megfelelo szintet
	public void init(int level2) {
		if (level2 != -1)
			this.level = level2;

		switch (level) {
		case 0:
			loadLevel0();
			break;
		case 1:
			loadLevel1();
			break;
		case 2:
			loadLevel2();
			break;
		case 3:
			loadLevel3();
			break;

		}
	}

	// elso szint
	private void loadLevel0() {
		for (int i = 1; i <= 27; i++) { // vizszintes utak elkszitese
			addNode(i + 100, 1230, (i - 1) * 30, 114, false); // 107-es
			// csomopont a
			// bal felso
			// keresztezodes
			// 122.es a jobb felso
			if (i != 25)
				addNode(i + 200, 1230, (i - 1) * 30, 432, false); // 207-es
			// csomopont
			// a bal
			// also
			// keresztezodes
			else
				addNode(i + 200, 1230, (i - 1) * 30, 432, true);
			// 222-es a jobb also
			if (i > 1) {
				connect(i + 100 - 1, i + 100, 1, 3);
				connect(i + 200 - 1, i + 200, 1, 3);
			}
		}

		for (int i = 1; i <= 4; i++) {
			addNode(i + 300, 1230, 182, (i - 1) * 30 - 5, false);
			if (i > 1)
				connect(i + 300 - 1, i + 300, 2, 0);
		}

		for (int i = 6; i <= 14; i++) {
			addNode(i + 300, 1230, 182, (i - 1) * 30, false);
			if (i > 6)
				connect(i + 300 - 1, i + 300, 2, 0);
		}

		for (int i = 16; i <= 20; i++) {
			addNode(i + 300, 1230, 182, (i - 1) * 30 + 10, false);
			if (i > 16)
				connect(i + 300 - 1, i + 300, 2, 0);
		}

		for (int i = 1; i <= 4; i++) {
			addNode(i + 400, 1230, 602, (i - 1) * 30 - 5, false);
			if (i > 1)
				connect(i + 400, i + 400 - 1, 0, 2);
		}

		for (int i = 6; i <= 14; i++) {
			addNode(i + 400, 1230, 602, (i - 1) * 30, false);
			if (i > 6)
				connect(i + 400, i + 400 - 1, 0, 2);
		}

		for (int i = 16; i <= 20; i++) {
			addNode(i + 400, 1230, 602, (i - 1) * 30 + 10, false);
			if (i > 16)
				connect(i + 400, i + 400 - 1, 0, 2);
		}

		connect(304, 107, 2, 0); // keresztezodsek sszektzgetse
		connect(107, 306, 2, 0);
		connect(314, 207, 2, 0);
		connect(207, 316, 2, 0);

		connect(121, 404, 0, 2);
		connect(406, 121, 0, 2);
		connect(221, 414, 0, 2);
		connect(416, 221, 0, 2);
	}

	// masodik szint
	private void loadLevel1() {

		for (int i = 1; i <= 28; i++) {

			// jobbrl balra meno utak
			if (i != 3 && i != 8 && i != 14 && i != 20 && i != 25)
				addNode(i, 1230, 812 - (i - 1) * 30, 65, false);
			else
				addNode(i, 0, 812 - (i - 1) * 30, 65, false);
			if (i > 1) {
				connect(i - 1, i, 3, 1);
			}

			if (i != 3 && i != 8 && i != 14 && i != 20 && i != 25)
				addNode(i + 56, 1230, 812 - (i - 1) * 30, 245, false);
			else
				addNode(i + 56, 0, 812 - (i - 1) * 30, 245, false);
			if (i > 1) {
				connect(i + 55, i + 56, 3, 1);
			}

			if (i != 27) {
				if (i != 3 && i != 8 && i != 14 && i != 20 && i != 25)
					addNode(i + 140, 1230, 812 - (i - 1) * 30, 545, false);
				else
					addNode(i + 140, 0, 812 - (i - 1) * 30, 545, false);
			} else
				addNode(i + 140, 1230, 812 - (i - 1) * 30, 545, true); // bzis
			if (i > 1) {
				connect(i + 139, i + 140, 3, 1);
			}

			// balrl jobbra meno utak
			if (i != 4 && i != 9 && i != 15 && i != 21 && i != 26)
				addNode(i + 28, 1230, 2 + (i - 1) * 30, 125, false);
			else
				addNode(i + 28, 0, 2 + (i - 1) * 30, 125, false);
			if (i > 1) {
				connect(i + 28 - 1, i + 28, 1, 3);
			}

			if (i != 4 && i != 9 && i != 15 && i != 21 && i != 26)
				addNode(i + 84, 1230, 2 + (i - 1) * 30, 362, false);
			else
				addNode(i + 84, 0, 2 + (i - 1) * 30, 362, false);
			if (i > 1) {
				connect(i + 83, i + 84, 1, 3);
			}

			if (i != 4 && i != 9 && i != 15 && i != 21 && i != 26)
				addNode(i + 112, 1230, 2 + (i - 1) * 30, 454, false);
			else
				addNode(i + 112, 0, 2 + (i - 1) * 30, 454, false);
			if (i > 1) {
				connect(i + 111, i + 112, 1, 3);
			}

			if (i < 23) {
				if (i != 3 && i != 6 && i != 9 && i != 13 && i != 17 && i != 19) {
					addNode(i + 200, 1230, 92, 605 - (i - 1) * 30, false);
					addNode(i + 300, 1230, 242, 605 - (i - 1) * 30, false);
					addNode(i + 500, 1230, 603, 605 - (i - 1) * 30, false);
					if (i > 1 && i != 4 && i != 7 && i != 10 && i != 14
							&& i != 18 && i != 20) {
						connect(i + 200 - 1, i + 200, 0, 2);
						connect(i + 300 - 1, i + 300, 0, 2);
						connect(i + 500 - 1, i + 500, 0, 2);
					}
				}

				if (i != 3 && i != 5 && i != 9 && i != 13 && i != 16 && i != 19) {
					addNode(i + 400, 1230, 422, 5 + (i - 1) * 30, false);
					addNode(i + 600, 1230, 752, 5 + (i - 1) * 30, false);
					if (i > 1 && i != 4 && i != 6 && i != 10 && i != 14
							&& i != 17 && i != 20) {

						connect(i + 400 - 1, i + 400, 2, 0);
						connect(i + 600 - 1, i + 600, 2, 0);
					}
				}
			}

		}

		// 500-as felfel t csompontok
		connect(502, 148, 0, 2);
		connect(148, 504, 0, 2);
		connect(505, 133, 0, 2);
		connect(133, 507, 0, 2);
		connect(508, 105, 0, 2);
		connect(105, 510, 0, 2);
		connect(512, 64, 0, 2);
		connect(64, 514, 0, 2);
		connect(516, 49, 0, 2);
		connect(49, 518, 0, 2);
		connect(518, 8, 0, 2);
		connect(8, 520, 0, 2);

		// 300-as felfel t csompontok
		connect(302, 160, 0, 2);
		connect(160, 304, 0, 2);
		connect(305, 121, 0, 2);
		connect(121, 307, 0, 2);
		connect(308, 93, 0, 2);
		connect(93, 310, 0, 2);
		connect(312, 76, 0, 2);
		connect(76, 314, 0, 2);
		connect(316, 37, 0, 2);
		connect(37, 318, 0, 2);
		connect(318, 20, 0, 2);
		connect(20, 320, 0, 2);

		// 200-as felfel t csompontok
		connect(202, 165, 0, 2);
		connect(165, 204, 0, 2);
		connect(205, 116, 0, 2);
		connect(116, 207, 0, 2);
		connect(208, 88, 0, 2);
		connect(88, 210, 0, 2);
		connect(212, 81, 0, 2);
		connect(81, 214, 0, 2);
		connect(216, 32, 0, 2);
		connect(32, 218, 0, 2);
		connect(218, 25, 0, 2);
		connect(25, 220, 0, 2);

		connect(602, 3, 2, 0);
		connect(3, 604, 2, 0);
		connect(604, 54, 2, 0);
		connect(54, 606, 2, 0);
		connect(608, 59, 2, 0);
		connect(59, 610, 2, 0);
		connect(612, 110, 2, 0);
		connect(110, 614, 2, 0);
		connect(615, 138, 2, 0);
		connect(138, 617, 2, 0);
		connect(618, 143, 2, 0);
		connect(143, 620, 2, 0);

		connect(402, 14, 2, 0);
		connect(14, 404, 2, 0);
		connect(404, 43, 2, 0);
		connect(43, 406, 2, 0);
		connect(408, 70, 2, 0);
		connect(70, 410, 2, 0);
		connect(412, 99, 2, 0);
		connect(99, 414, 2, 0);
		connect(415, 127, 2, 0);
		connect(127, 417, 2, 0);
		connect(418, 154, 2, 0);
		connect(154, 420, 2, 0);
	}

	// harmadik szint
	private void loadLevel2() {
		for (int i = 1; i <= 27; i++) {
			if (i < 25) // felso vizszintes ut
			{
				addNode(i + 100, 1230, 52 + i * 30, 33, false);
				if (i > 1) {
					connect(i + 100 - 1, i + 100, 1, 3);
				}
			}

			addNode(i + 200, 1230, 802 - i * 30, 483, false); // also vizszintes
			// ut
			if (i > 1) {
				connect(i + 200 - 1, i + 200, 3, 1);
			}

			if (i < 20 && i != 3 && i != 18 && i != 10) {
				addNode(i + 300, 1230, 82, 573 - i * 30, false);
				if (i > 1 && i != 4 && i != 19 && i != 11) {
					connect(i + 300 - 1, i + 300, 0, 2);
				}
			}

			if (i < 15 && i != 12 && i != 5) {
				addNode(i + 400, 1230, 172, 153 + (i - 1) * 30, false);
				if (i > 1 && i != 13 && i != 6) {
					connect(i + 400 - 1, i + 400, 2, 0);
				}
			}

			if (i < 5 && i != 3) {
				addNode(i + 420, 1230, 172, 95 - (i - 1) * 30, false);
				if (i > 1 && i != 4) {
					connect(i + 420 - 1, i + 420, 0, 2);
				}
			}

			if (i < 8) {
				if (i != 4 && i != 7) {
					if (i < 4)
						addNode(i + 500, 1230, (i - 1) * 30 - 8, 273, true);
					else
						addNode(i + 500, 1230, (i - 1) * 30 - 8, 273, false);
				} else
					addNode(i + 500, 0, (i - 1) * 30 - 8, 273, false); // LMPS!!!!
				if (i > 1)
					connect(i + 500 - 1, i + 500, 1, 3);
			}

			if (i < 8) {
				addNode(i + 600, 1230, 352 - (i - 1) * 30, 126, false);
				if (i > 1)
					connect(i + 600 - 1, i + 600, 3, 1);
			}

			if (i < 15) {
				if (i != 12) {
					addNode(i + 700, 1230, 352, 483 - i * 30, false);
					if (i > 1 && i != 13)
						connect(i + 700 - 1, i + 700, 0, 2);
				}
			}

			if (i < 6) {
				addNode(i + 720, 1230, 202 + (i - 1) * 30, 363, false);
				if (i > 1) {
					connect(i + 720 - 1, i + 720, 1, 3);
				}
			}

			if (i < 7) {
				addNode(i + 1000, 1230, 622 + (i - 1) * 30, 422, false);
				if (i > 1) {
					connect(i + 1000 - 1, i + 1000, 1, 3);
				}
			}

			if (i < 22 && i != 2 && i != 17 && i != 15) {
				addNode(i + 800, 1230, 622, (i - 1) * 30, false);
				addNode(i + 900, 1230, 742, (i - 1) * 30, false);
				if (i > 1 && i != 3 && i != 18 && i != 16) {
					connect(i + 800 - 1, i + 800, 2, 0);
					connect(i + 900 - 1, i + 900, 2, 0);
				}
			}

		}

		addNode(425, 1230, 142, 543, false);
		addNode(426, 1230, 112, 543, false);
		connect(425, 426, 3, 1);
		connect(414, 425, 3, 1);
		connect(426, 301, 3, 1);

		connect(302, 224, 0, 2);
		connect(224, 304, 0, 2);
		connect(317, 101, 0, 2);
		connect(101, 319, 0, 2);
		connect(422, 104, 0, 2);
		connect(104, 424, 0, 2);
		connect(411, 221, 2, 0);
		connect(221, 413, 2, 0);
		connect(309, 504, 0, 2);
		connect(504, 311, 0, 2);
		connect(404, 507, 2, 0);
		connect(507, 406, 2, 0);
		connect(607, 421, 0, 2);
		connect(607, 401, 2, 0);
		connect(711, 601, 0, 2);
		connect(601, 713, 0, 2);
		connect(408, 721, 1, 3);
		connect(725, 704, 1, 3);
		connect(215, 701, 0, 2);
		connect(814, 1001, 2, 0);
		connect(1001, 816, 2, 0);
		connect(816, 206, 2, 0);
		connect(206, 818, 2, 0);
		connect(801, 119, 2, 0);
		connect(119, 803, 2, 0);
		connect(914, 1005, 2, 0);
		connect(1005, 916, 2, 0);
		connect(916, 202, 2, 0);
		connect(202, 918, 2, 0);
		connect(901, 123, 2, 0);
		connect(123, 903, 2, 0);
		connect(714, 110, 0, 2);

	}

	// negyedik szint
	private void loadLevel3() {
		for (int i = 1; i <= 28; i++) {
			if (i != 25)
				addNode(i + 100, 1230, 805 - (i - 1) * 30, 54, false); // felso
			// hosszu
			// vizszintes
			// t
			else
				addNode(i + 100, 0, 805 - (i - 1) * 30, 54, false); // !!!!!!!!LAMPAS
			// KERESZTEZODES!!!!!!!!!

			if (i != 3)
				addNode(i + 200, 1230, 805 - (i - 1) * 30, 534, false); // also
			// hossz
			// vizszintes
			// ut
			else
				addNode(i + 200, 0, 805 - (i - 1) * 30, 534, false); // !!!!!!!!LAMPAS
			// KERESZTEZODES!!!!!!!!!

			if (i > 1) {
				connect(i + 100 - 1, i + 100, 3, 1);
				connect(i + 200 - 1, i + 200, 3, 1);
			}

			if (i < 16) // 1. fuggoleges ut (felfele)
			{
				if (i != 12)
					addNode(i + 300, 1230, 85, 534 - i * 30, false);
				else
					addNode(i + 300, 1230, 85, 534 - i * 30, true);
				if (i > 1)
					connect(i + 300 - 1, i + 300, 0, 2);
			}

			// innentol spiralban haladok
			if (i < 14) // 2. fuggoleges ut (lefele)
			{
				addNode(i + 400, 1230, 175, 54 + i * 30, false);
				if (i > 1)
					connect(i + 400 - 1, i + 400, 2, 0);
			}

			if (i < 19) {
				addNode(i + 500, 1230, 175 + (i - 1) * 30, 474, false);
				if (i > 1) {
					connect(i + 500 - 1, i + 500, 1, 3);
				}
			}

			if (i < 10) {
				addNode(i + 600, 1230, 685, 474 - i * 30, false);
				if (i > 1) {
					connect(i + 600 - 1, i + 600, 0, 2);
				}
			}

			if (i < 11) {
				addNode(i + 700, 1230, 685 - i * 30, 203, false);
				if (i > 1) {
					connect(i + 700 - 1, i + 700, 3, 1);
				}
			}

			if (i < 5) {
				addNode(i + 800, 1230, 385, 203 + i * 30, false);
				if (i > 1) {
					connect(i + 800 - 1, i + 800, 2, 0);
				}
			}

			if (i < 5) {
				addNode(i + 900, 1230, 385 + i * 30, 323, false);
				if (i > 1) {
					connect(i + 900 - 1, i + 900, 1, 3);
				}
			}

			if (i < 3) {
				addNode(i + 1000, 1230, 504, 323 - i * 30, false);
				if (i > 1) {
					connect(i + 1000 - 1, i + 1000, 0, 2);
				}
			}

			if (i < 5) {
				addNode(i + 1100, 1230, 504 + i * 30, 263, false);
				if (i > 1) {
					connect(i + 1100 - 1, i + 1100, 1, 3);
				}
			}

			if (i < 5) {
				addNode(i + 1200, 1230, 624, 263 + i * 30, false);
				if (i > 1) {
					connect(i + 1200 - 1, i + 1200, 2, 0);
				}
			}

			if (i < 12) {
				addNode(i + 1300, 1230, 624 - i * 30, 383, false);
				if (i > 1) {
					connect(i + 1300 - 1, i + 1300, 3, 1);
				}
			}

			if (i < 9) {
				addNode(i + 1400, 1230, 294, 383 - i * 30, false);
				if (i > 1) {
					connect(i + 1400 - 1, i + 1400, 0, 2);
				}
			}

			if (i < 16) {
				addNode(i + 1500, 1230, 295 + i * 30, 143, false);
				if (i > 1) {
					connect(i + 1500 - 1, i + 1500, 1, 3);
				}
			}

			if (i < 13) {
				addNode(i + 1600, 1230, 745, 142 + i * 30, false);
				if (i > 1) {
					connect(i + 1600 - 1, i + 1600, 2, 0);
				}
			}

		}

		connect(413, 501, 2, 0);
		connect(518, 601, 0, 2);
		connect(609, 701, 3, 1);
		connect(710, 801, 2, 0);
		connect(804, 901, 1, 3);
		connect(904, 1001, 0, 2);
		connect(1002, 1101, 1, 3);
		connect(1104, 1201, 2, 0);
		connect(1204, 1301, 3, 1);
		connect(1311, 1401, 0, 2);
		connect(1408, 1501, 1, 3);
		connect(1515, 1601, 2, 0);
		connect(1612, 203, 2, 0);
		connect(225, 301, 0, 2);
		connect(122, 401, 2, 0);
		connect(315, 125, 0, 2);

	}

	// Uj nyulat ad a palyahoz
	public void newBunny(Bunny bunny) {
		LinkedList<Node> emptyNodes = getEmptyNodes();
		if (emptyNodes.size() > 0) {
			Node n = emptyNodes.get(Game.random.nextInt(emptyNodes.size()));
			n.setCar(bunny);
			cars.add(bunny);
			bunny.setNode(n);
		}
	}

	// Uj autot ad a palyahoz
	public void newCar(Car c) {
		LinkedList<Node> entryNodes = getEntryNodes();
		Collections.shuffle(entryNodes);
		for (Node n : entryNodes) {
			if (n.getCar() == null) {
				n.setCar(c);
				c.setNode(n);
				cars.add(c);
				break;
			}
		}
	}

	// Rabloautot tesz a palyara
	public void newRunnerCar(RunnerCar c) {
		runnerCar = c;
		switch (level) {
		case 0:
			nodes.get(201).setCar(c);
			c.setNode(nodes.get(201));
			break;
		case 1:
			nodes.get(5).setCar(c);
			c.setNode(nodes.get(5));
			break;
		case 2:
			nodes.get(807).setCar(c);
			c.setNode(nodes.get(807));
			break;
		case 3:
			nodes.get(403).setCar(c);
			c.setNode(nodes.get(403));
			break;
		}
	}

	// Kiveszi a rablok autojat a palyarol
	public void removeCar(Car c) {
		cars.remove(c);
	}

	// Kitoroli a rablok autojat a palyarol
	public void removeRunnerCar() {
		runnerCar = null;
	}

	// Tick
	@SuppressWarnings("unchecked")
	public void tick() {
		synchronized (nodes) {
			Collection<Node> nodesTemp = nodes.values();
			for (Node e : nodesTemp) {
				e.step();
			}
		}
		LinkedList<Car> carsTemp;
		synchronized (cars) {
			carsTemp = (LinkedList<Car>) cars.clone();
		}
		for (Car c : carsTemp) {
			c.step();
		}
	}
}
