/* 
 * Just Another Team
 */

package model;

import java.util.Random;
import view.View;

// Game osztaly
public class Game {

	public static final int PLAYER_LOST = 3;

	public static final int PLAYER_WIN = 4;

	public static final int PLAYING = 2;

	// Veletlen generator
	public static Random random;

	// Kivulrol is elerheto konstansok
	public static final int WAITING_FOR_RUNNERCAR = 1;

	// CarFactory-ra mutato referencia
	private CarFactory carFactory;

	// City-re mutato referencia
	private City city;

	// Jatek allapota/allasa
	private int gameState = 0;
	// Timer-re mutato referencia
	private Timer timer;
	// Idozitesert felelos szal
	private Thread timerThread;
	// View-ra mutato referencia
	private View view;

	// Konstruktor
	public Game(View view) {

		// Beallitjuk a belso referenciat
		this.view = view;

		// Inicializaljuk a veletlen generatort
		random = new Random();

		// Inicializaljuk a varost
		city = new City(this);

		// Inicializaljuk a carfactoryt
		carFactory = new CarFactory();

		// Beallitjuk a carfactoryt
		carFactory.setCity(this, city);

		// Jatek kezdeti allapota
		gameState = WAITING_FOR_RUNNERCAR;

		// Inicializalja az idozitot
		timer = new Timer();

		// Beallitjuk az idozitot
		timer.setGame(this);
		timer.setPeriod(100);

		// Beallitjuk a szalat
		timerThread = new Thread(timer);
		timerThread.setDaemon(true);

		// Letrehozzuk a varos megjelenitesert felelos osztalyt
		view.createCityView(city);

		// Inicializaljuk a varost a 0-as palyara
		city.init(0);

		// Lepunk nehanyat
		for (int i = 0; i < 3000; i++)
			tick();

		// Beinditjuk a szalat
		timerThread.start();
	}

	// A jatekos vesztett
	public void endGame() {
		gameState = PLAYER_LOST;
	}

	// Getter fuggveny belso tagvaltozohoz
	public int getGameState() {
		return gameState;
	}

	// Getter fuggveny belso tagvaltozohoz
	public View getView() {
		return view;
	}

	// Palya betoltese
	public void loadLevel(int level) {
		// Ujrainditjuk a megjelenitesert felelos osztalyt
		view.reset();

		// Letrehozzuk az uj varost
		city = new City(this);

		// Atallitjuk carfactoryt
		carFactory.setCity(this, city);

		// Beallitjuk a jatek allapotat
		gameState = WAITING_FOR_RUNNERCAR;

		// Letrehozzuk a varos megjelenitesert felelos osztalyt
		view.createCityView(city);

		// Inicializaljuk a varost a megfelelo palyara
		city.init(level);

		// Lepunk nehanyat
		for (int i = 0; i < 3000; i++)
			tick();
	}

	// Lepteti a rablok autojat a megfelelo iranyba
	public void Move(int i) {
		// Ha meg nincs meg a rablok autoja, akkor elindul a jatek
		if (city.getRunnerCar() != null)
			city.getRunnerCar().step(i);
		else
			startGame();
	}

	// Szuntelteti a jatekot
	public void pause() {
		// Megszakitjuk az idozito szalat
		if (timerThread.isAlive())
			timerThread.interrupt();
		else {
			// Ha nem futott, akkor ujrainditja
			timer = new Timer();
			timer.setGame(this);
			timer.setPeriod(100);
			timerThread = new Thread(timer);
			timerThread.setDaemon(true);
			timerThread.start();
		}
	}

	// Elinditja a jatekot
	public void startGame() {
		// Ha meg nem indult el
		if (gameState == WAITING_FOR_RUNNERCAR) {
			// Megalkotjuk a rablok autojat
			carFactory.newRunnerCar();
			gameState = PLAYING;
		}
	}

	// Mozgatja a varos megfelelo komponenseit
	public void tick() {
		city.tick();
		carFactory.tick();
		view.repaint();
	}

	// A jatekos nyert
	public void winGame() {
		gameState = PLAYER_WIN;
	}
}
