/* 
 * Just Another Team
 */

package model;

import java.util.LinkedList;

// Node osztaly
public class Node {

	// Tartalmazott auto
	private Car car;

	// Varosra mutato referencia
	private City city;

	// Rejtekhely?
	private boolean hiding;

	// Azonosito
	private int id;

	// Bejovo csomopontok
	private Node[] inNodes;

	// A jelzolampa kovetkezo allasa
	private int nextTrafficLight;

	// Kimeno csomopontok
	private Node[] outNodes;

	// Elsobbsegi sorrend
	private LinkedList<Integer> priority;

	// Hanyat lepett mar a jelzolampa
	private int step;

	// Jelzolampa
	private int trafficLight;

	// x iranybeli pozicio
	private int x;

	// y iranybeli pozicio
	private int y;

	// Konstruktor
	public Node(City city, int id, int type, boolean hiding) {

		// Meg nem volt lepes
		step = 0;

		// Azonositot beallitjuk
		this.id = id;

		// Varosra mutato referencia beallitasa
		this.city = city;

		// rejtekhely?
		this.hiding = hiding;

		// Bejovo kapcsolatoknak foglalunk helyet
		inNodes = new Node[4];

		// Kimeno kapcsolatoknak foglalunk helyet
		outNodes = new Node[4];

		// Jelzolampa?
		if (type == 0) {
			trafficLight = 0;
			nextTrafficLight = -1;
			priority = null;
		} else {
			// beallitjuk a prioritas sorrendjet
			trafficLight = -1;
			priority = new LinkedList<Integer>();
			priority.add(type % 10000 / 1000);
			priority.add(type % 1000 / 100);
			priority.add(type % 100 / 10);
			priority.add(type % 10);
		}
	}

	// Lekerdezi a bejovo kapcsolatok szamat
	public int activeInNodes() {
		int db = 0;
		// Vegigmegyunk az osszes bejovo kapcsolaton
		for (int i = 0; i < 4; i++) {
			if (inNodes[i] != null)
				db++;
		}
		return db;
	}

	// Lekerdezi a kimeno kapcsolatok szamat
	public int activeOutNodes() {
		int db = 0;
		// Vegigmegyunk az osszes kimeno kapcsolaton
		for (int i = 0; i < 4; i++) {
			if (outNodes[i] != null)
				db++;
		}
		return db;
	}

	// Osszekapcsol ket csomopontot
	public void connect(Boolean in, int dir, Node n) {
		if (in)
			inNodes[dir] = n;
		else
			outNodes[dir] = n;
	}

	// Getter fuggveny belso tagvaltozohoz
	public Car getCar() {
		return car;
	}

	// Getter fuggveny belso tagvaltozohoz
	public City getCity() {
		return city;
	}

	// Megtalalja adott csomoponthoz tartozo iranyt
	public int getDir(Node node) {
		// Vegignezzuk mind a 4 kapcsolatot
		for (int i = 0; i < 4; i++) {
			if (inNodes[i] == node)
				return i;
		}
		// Nem talaltuk meg
		return -1;
	}

	public Integer getId() {
		return id;
	}

	// Getter fuggveny belso tagvaltozohoz
	public Node[] getInNodes() {
		return inNodes;
	}

	// Getter fuggveny belso tagvaltozohoz
	public Node[] getOutNodes() {
		return outNodes;
	}

	// Getter fuggveny belso tagvaltozohoz
	public LinkedList<Integer> getPriority() {
		return priority;
	}

	// Getter fuggveny belso tagvaltozohoz
	public int getTrafficLight() {
		return trafficLight;
	}

	// Getter fuggveny belso tagvaltozohoz
	public int getType() {
		if (priority == null)
			return 0;
		else
			return priority.get(0) * 1000 + priority.get(1) * 100
					+ priority.get(2) * 10 + priority.get(3);
	}

	public int getX() {
		return x;
	}

	public int getY() {
		return y;
	}

	// Getter fuggveny belso tagvaltozohoz
	public boolean isHiding() {
		return hiding;
	}

	// Setter fuggveny belso tagvaltozohoz
	public void setCar(Car car) {
		this.car = car;
	}

	// Setter fuggveny belso tagvaltozohoz
	public void setTrafficLight(int dir) {
		nextTrafficLight = dir;
		step();
	}

	// Lepteti a jelzolampat
	public void step() {
		// Lepes
		step++;
		// 3 masodpercenkent atall a jelzolampa
		if (step % 30 == 0) {
			// Ha jelzolampas
			if (trafficLight != -1) {
				if (nextTrafficLight != -1) {
					// Dontottunk mar a kovetkezo allasrol
					trafficLight = nextTrafficLight;
					nextTrafficLight = -1;
				} else {
					// Most dontunk a kovetkezo allapotrol
					if (activeInNodes() > 0)
						trafficLight = (trafficLight + 1) % 4;
					while (inNodes[trafficLight] == null) {
						trafficLight = (trafficLight + 1) % 4;
					}
				}
			}
		}
	}
}
