/* 
 * Just Another Team
 */

package view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.util.LinkedList;
import java.util.TreeMap;

import javax.swing.JPanel;

import model.Bunny;
import model.Car;
import model.Game;
import model.Node;
import model.PoliceCar;
import model.RunnerCar;
import model.City;

//View osztaly
public class View extends JPanel {

	private static final long serialVersionUID = 1L;
	private LinkedList<Paintable> components;
	// Belso allapot/referenciak
	private Game game;
	private TreeMap<Integer, NodeView> nodeViews;

	// Konstrukor
	public View() {
		super();
		// Inicializaljuk az adatstrukturakat
		components = new LinkedList<Paintable>();
		nodeViews = new TreeMap<Integer, NodeView>();
	}

	// Letrehoz egy megjelenito objektumot
	public void createBunnyView(Bunny c) {
		components.addLast(new BunnyView(c, this));
	}

	// Letrehoz egy megjelenito objektumot
	public void createCarView(Car c) {
		components.addLast(new CarView(c, this, Game.random.nextInt(7)));
	}

	// Letrehoz egy megjelenito objektumot
	public void createCityView(City city) {
		components.addFirst(new CityView(city, this, 1));
	}

	// Letrehoz egy megjelenito objektumot
	public void createNodeView(Node t, int x, int y) {
		NodeView nodeView = new NodeView(t, this, x, y);
		components.addLast(nodeView);
		// Eltaroljuk, hogy elerheto legyen az autok szamara
		nodeViews.put(t.getId(), nodeView);
	}

	// Letrehoz egy megjelenito objektumot
	public void createPoliceCarView(PoliceCar c) {
		components.addLast(new PoliceCarView(c, this));
	}

	// Letrehoz egy megjelenito objektumot
	public void createRunnerCarView(RunnerCar c) {
		components.addLast(new RunnerCarView(c, this));
	}

	// Getter fuggveny belso tagvaltozohoz
	public Game getGame() {
		return game;
	}

	// Getter fuggveny belso tagvaltozohoz
	public TreeMap<Integer, NodeView> getNodeViews() {
		return nodeViews;
	}

	// Getter fuggveny belso tagvaltozohoz
	@Override
	public Dimension getPreferredSize() {
		// Ablak merete
		return new Dimension(800, 600);
	}

	// Kirajzolja a komponenseket
	@Override
	@SuppressWarnings("unchecked")
	protected void paintComponent(Graphics g) {
		LinkedList<Paintable> temp;
		synchronized (components) {
			temp = (LinkedList<Paintable>) components.clone();
		}

		// Kirajzolja az osszes komponenst
		for (Paintable e : temp) {
			Graphics2D g2 = (Graphics2D) g.create();
			e.paint(g2);
		}

		// nyeres eseten kiirja
		if (game.getGameState() == Game.PLAYER_WIN) {
			g.setColor(new Color(1f, 1f, 1f, 0.9f));
			g.setFont(new Font("TimesRoman", Font.BOLD, 30));
			g.drawString("Gratullok, nyertl!", 300, 60);
		}

		// Vesztes eseten kiirja
		if (game.getGameState() == Game.PLAYER_LOST) {
			g.setColor(new Color(1f, 1f, 1f, 0.9f));
			g.setFont(new Font("TimesRoman", Font.BOLD, 30));
			g.drawString("Sajnos vesztettl!", 300, 60);
		}
	}

	// Ujra inicializalja az osztalyt
	public void reset() {
		components = new LinkedList<Paintable>();
		nodeViews = new TreeMap<Integer, NodeView>();
	}

	// Setter fuggveny belso tagvaltozohoz
	public void setGame(Game game) {
		this.game = game;
	}
}
